MISmat <-
function(dser,ichd=0,missing=NA,outfile=NULL)
{
# Initialisation
dser[dser==missing]<-NA
n<-nrow(dser)
m<-ncol(dser)
MISmat<-list(title="MISmat: Missing data estimates by MAT method")
MISmat$rmse<-rep(NA,m)
MISmat$R2<-rep(NA,m)
MISmat$r<-rep(NA,m)
MISmat$misnumber<-rep(0,m)
MISmat$correction<-rep(0,m)
MISmat$estim<-matrix(NA,n,m*3)
MISmat$dser<-matrix(NA,n,m)
MISmat$nbanal<-matrix(0,n)
MISmat$mindist<-matrix(NA,n)
MISmat$maxdist<-matrix(NA,n)

if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}
cat("MISSING DATA : estimates by Modern analogues technique (MAT)\n")
cat("----------------------------------------------------------------------\n")
mdist<-"euclidian"
if(ichd==3) {mdist="canberra"}

#Display input parameters in a file
cat("distance option (0=euclidian /1=chord /2=log /3=canberra) = ",ichd,"\n")

# beginning of analogue search for the calibration data
bac <- bestanal_NA (dser,ichd) 
ye <- WANA_reconstruction (dser,bac$dist,bac$list,kana=0,2*bac$seuil)

#loop on the m variables
MISmat$dser<-dser
for (j in 1:m) {
	cat("\nVariable : ",colnames(dser)[j],"\n")
	cat("-----------------------------------\n")
	jj<-(j-1)*3
	moy1=mean(dser[,j],na.rm=T)
	moy2=mean(ye$rmean[!is.na(dser[,j]),j],na.rm=TRUE)
	ect1=sd(dser[,j],na.rm=TRUE)
	ect2=sd(ye$rmean[!is.na(dser[,j]),j],na.rm=TRUE)
	MISmat$correction[j]=ect1/ect2
	if (MISmat$correction[j]<1.2) {MISmat$correction[j]=1}
	MISmat$estim[,jj+1]<-(ye$rmean[,j]-moy2)*MISmat$correction[j]+moy1
	MISmat$estim[,jj+2]<-(ye$rmean[,j]-ye$rmin[,j])*MISmat$correction[j]
	MISmat$estim[,jj+3]<-(ye$rmax[,j]-ye$rmean[,j])*MISmat$correction[j]
	MISmat$misnumber[j]<-sum(is.na(dser[,j]))
	if (j==1) {
		MISmat$nbanal<-ye$nbanal
		MISmat$mindist<-ye$mindist
		MISmat$maxdist<-ye$maxdist
	}
	MISmat$rmse[j]<-mean((dser[,j]-ye$rmean[,j])^2,na.rm=TRUE)^0.5
	MISmat$R2[j]<-1-(MISmat$rmse[j]/sd(dser[,j],na.rm=TRUE))^2
	MISmat$r[j]<-cor(dser[,j],MISmat$estim[,jj+1], use="pairwise.complete.obs")
	cat("r = ",MISmat$r[j]," -- R2 = ",MISmat$R2[j],"  -- RMSE = ",MISmat$rmse[j],"calculated on ",n-MISmat$misnumber[j],"  observations\n")
	cat("correction factor (ratio of std.dev if >1.2) = ",MISmat$correction[j],"  - Std.Dev = ",ect1,ect2,"\n")
	plot(dser[,j],MISmat$estim[,jj+1],xlab='obs', ylab='est',main=colnames(dser)[j])
	abline(0,1,col="red")
	for (i in 1:n) {
		if (is.na(dser[i,j])) {MISmat$dser[i,j]<-MISmat$estim[i,jj+1]}
	}
}
coln<<-rep(NA,m*3)
for (j in 1:m) {
	coln[(j-1)*3+1]<-colnames(dser)[j]
	coln[(j-1)*3+2]<-paste(colnames(dser)[j],'_i',sep="")
	coln[(j-1)*3+3]<-paste(colnames(dser)[j],'_s',sep="")
}
colnames(MISmat$estim)<-coln
colnames(MISmat$dser)<-colnames(dser)
rownames(MISmat$estim)<-rownames(dser)	
rownames(MISmat$dser)<-rownames(dser)	
if (!is.null(outfile)) {
	cat("Completed matrix\n",file=ftext,append=TRUE)
	cat("The estimated values have been corrected by the ratio of standard deviations\n",file=ftext,append=TRUE)
	write.table(MISmat$dser,file=ftext,append=TRUE)
	cat("Full estimates with the (min-max) range (no correction)\n", file=ftext,append=TRUE)
	write.table(MISmat$estim,file=ftext,append=TRUE)
	cat("Number of analogues for each observation/variable\n",file=ftext,append=TRUE)
	write.table(MISmat$nbanal,file=ftext,append=TRUE)
	dev.off()
	sink()
}

return(MISmat)
}
