cartosurfi <-
function(lon,lat,champ,Uind,maxUind=NA,limits=NA,zlim=NA,coldir=1,name="",screen=TRUE)
# lon, lat two vector of longitudes and latitudes for plotting on the map
# champ = vector of z-values to plot with color code on the map
# plot a map with coloured points of size inversely proportionnal to uncertainty Uind (0 = very precise)
# Uind = index of uncertainty same size than champ (positive values)
# maxUind = max value of uncertainty: all values >maxUind are set to maxUind
# if coldir=1: the palette goes from blue to red                               
# if coldir=-1: the palette goes from red to blue                              
# if coldir=2: the palette goes from blue to red through white                               
# if coldir=-2: the palette goes from red to blue through white                           
# limits[4] = vector of limits of the map (minx, maxx, miny, maxy)             
# 	if this vector is NULL, they are calculated on the data (lon, lat)           
# zlim = two-values of min and max for champ
# name for the title of the graphics and for the graphic name if screen=FALSE
# screen=TRUE graphics on the screen, otherwise on a pdf file
{

# elimination des données manquantes
lon=lon[!is.na(champ)]
lat=lat[!is.na(champ)]
champ=champ[!is.na(champ)]
Uind=Uind[!is.na(champ)]
if (is.na(maxUind)) {maxUind=max(Uind)}
Uind[Uind>maxUind]=maxUind

# initialisation
if (is.na(limits[1])) {
    minx<-min(lon)
    miny<-min(lat)
    maxx<-max(lon)
    maxy<-max(lat)
} else {
    minx<-limits[1]
    maxx<-limits[2]  
    miny<-limits[3]   
    maxy<-limits[4]
}
if (is.na(zlim[1])) {
    minz<-min(champ)
    maxz<-max(champ)
    zlim=c(minz,maxz)
} else {
    minz<-zlim[1]
    maxz<-zlim[2]  
}

# couleurs
ncolor<-50
if (!screen) pdf(file=paste("map_",name,".pdf",sep=""))
#op<-par(mai=c(0.4, 0.4, 0.5, 0.2))
if (abs(coldir)==1) {coli<-rainbow(ncolor,start=0,end=0.8)} 
if (abs(coldir)==2) {coli<-colorRampPalette(c("dark red","red","orange","white","white","cyan","blue","dark blue"))(ncolor)}
indx<-seq(ncolor,1,by=-1)
cold<-coli[indx]
if (coldir>=0) {cols=cold}
else {cols=coli}

# calculate surface
fit<-Tps(cbind(lon,lat),champ)
out.p=predict.surface(fit)
out.p$z[out.p$z>maxz]=maxz
out.p$z[out.p$z<minz]=minz


# draw points
xmoy=champ
xmoy[xmoy>maxz]=maxz
xmoy[xmoy<minz]=minz
qq=seq(zlim[1],zlim[2],length=length(cols))
l.col<-c()
cex=rep(0.75,length(champ))
for(i in 1:length(champ)) {
	l.col=c(l.col,which(xmoy[i]>=qq[1:(length(qq)-1)] & xmoy[i]<=qq[2:length(qq)]))
  if(abs(champ[i])>Uind[i]) {cex[i]=1.25}
	if(abs(champ[i])>2*Uind[i]) {cex[i]=2}
	if(abs(champ[i])>3*Uind[i]) {cex[i]=2.5}
}
surface(out.p,type="I",xlab="",ylab="", main=name,ylim=c(miny,maxy),xlim=c(minx,maxx), zlim=c(minz,maxz), graphics.reset=T,col=cols)
points(lon,lat,cex=cex,col="black",bg=cols[l.col],pch=21)
world(ylim=c(miny,maxy),xlim=c(minx,maxx),add=TRUE)
#image.plot(xmoy,col=cols,legend.only=TRUE, graphics.reset=T,zlim=range(qq))
#par(op)	
message="well done"
if (!screen) dev.off()
return(message)
}
