linreg <-
function(xmatname,ymatname,outfile=NULL)
# this function calculates regression coefficients between matrix y and matrix x
# there is no prediction but it uses a stepwise regression
{
# Initialisation

library(MASS)
if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}

x<-as.data.frame(xmatname)
y<-as.data.frame(ymatname)
xobsname<-rownames(x)
yobsname<-rownames(y)
nx<-nrow(x)
mx<-ncol(x)
ny<-nrow(y)
my<-ncol(y)
if (identical(ny,nx)==FALSE) stop ("both matrices have not same number of rows")

# mean and standard deviation

xmean<-apply(x,2,mean)
ymean<-apply(y,2,mean)
xstd<-apply(x,2,sd)
ystd<-apply(y,2,sd)

cat("\tPredictand reconstruction using a linear model\n\tSummary statistics of X ref-data :\n")
print(summary(x))
cat("\n\tSummary statistics of Y ref-data :\n")
print(summary(y))

# loop on dependent variables

ng <- min(round(sqrt(my)),4)   # max 16 graphics
ng2<-ng^2
opold<-par(no.readonly = TRUE)
op<-par(cex.main=2,cex.axis=1.2,cex.lab=1.3)

# linear model

X <- as.matrix(x)

#Modif GY 25/08/08
#Z <- y
Z <- as.matrix(y)

R2<-rep(0,my)
kgraph<-0
for (j in 1:my)
	{
	cat("\n\tDependent Variable : ",colnames(y)[j])	
	Y <- y[,j]
	LMY0<-lm(Y~1)
	LMY<-lm(Y~X)
	summa0<-summary(LMY0)
	summa<-summary(LMY)
	cat("\n\n\tSummary LMY : ")
	print(summa)
	SLMY<-step(LMY0,scope = list(lower = formula(LMY0), upper = formula(LMY)))
	summas<-summary(SLMY)
	cat("\n\tSummary SLMY0 : ")
	print(summa0)
	cat("\n\tSummary SLMY : ")
	print(summas)
	Z[,j] <- fitted.values(SLMY)
	plot(Z[,j],Y, type='p',xlab='Pred', ylab='Obs', main=colnames(y)[j])
	lines(c(min(Y),max(Y)),c(min(Y),max(Y)),col="green")
	R2[j] <- summary(SLMY)$r.squared 
	mod=j-round(j/ng2)*ng2
	}
R2tot <- mean(R2)
cat("\tValeurs des R2 :\n")
for (j in 1:my) { cat(colnames(y)[j],R2[j],"\n") }
cat("R2tot = ",R2tot,"\n")
if (!is.null(outfile))
	{
	dev.off()
	sink()
	}
return(Z)
}
