predLDb <-
function(xmat,catmat,xnewmat,catnewmat=FALSE,p=0,ncas=100,xstep=1,outfile=NULL)
{

# Initialisation
if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}
catmat=as.vector(catmat)
na<-nrow(xmat)
ma<-ncol(xmat)
nc<-length(catmat)
nfa<-nrow(xnewmat)
mfa<-ncol(xnewmat)
if (identical(na,nc)==FALSE) {stop ("both xmat & catmat matrices have not same number of rows")}
if (identical(mfa,ma)==FALSE) {stop ("both xmat & xnewmat matrices have not same number of columns")}
ng=length(unique(catmat))
nfc<-0
mfc<-0
option="extrapolation"
LDB<-list(title="Linear discrimant analysis by bootstrap method")
LDB$ng<-ng
ng3=ng*3
LDB$class0<-matrix(0,ng,ng)
LDB$clas<-matrix(0,ng,ng)
LDB$class<-array(0,c(ng,ng,ncas))
LDB$classp<-array(0,c(ng,ng,ncas))
LDB$catprob=as.data.frame(matrix(NA,na,(ng3+1)))
LDB$catnewprob=as.data.frame(matrix(NA,na,(ng3+1)))
print(catnewmat)
if (is.logical(catnewmat)==FALSE) {
	nfc<-length(catnewmat)
	if (identical(nfc,nfa)==FALSE) stop ("both catnewmat & xnewmat matrices have not same number of rows")
	option="verification"
	LDB$clasp<-matrix(0,ng,ng)
	LDB$catnewprob=as.data.frame(matrix(NA,nfa,(ng3+1)))
	catnewmat=as.vector(catnewmat)
}

ncal<-na*4/5
Yec<-array(NA,c(nc,ng,ncas))
Yev<-array(NA,c(nc,ng,ncas))
Zec<-array(NA,c(nfa,ng,ncas))
Ye3<-array(NA,c(3,nc,ng))
Ze3<-array(NA,c(3,nfa,ng))

cat("DISCRIMINANT ANALYSIS : LDB method\n")
cat("--------------------------------------------------------------\n")
cat("\nOption : ",option,"\n")
cat("if option is extrapolation, verification is done on the observations not selected by the bootstrap procedure\n")
cat("if option is verification, verification is done on the new data (xnewdata,catnewmat)\n")

# transformation in pc if p>0

if (p>0) {
	pc.xmat<-prcomp(xmat,scale.=TRUE)
	screeplot(pc.xmat, npcs=p, type="barplot",main="Screeplot of the eignevalues")
	biplot(pc.xmat)
	z.xmat<-predict(pc.xmat,xmat)
	z.xnewmat<-predict(pc.xmat,xnewmat)
	print(summary(pc.xmat))
} else {
	z.xmat<-xmat
	z.xnewmat<-xnewmat
	p<-ma
}

# standardization
X<-as.matrix(z.xmat[,1:p])
Z<-as.matrix(z.xnewmat[,1:p])

# estimation on all the data
dd.ld <- lda(X,t(catmat))
grsize=dd.ld$counts 
print(dd.ld)
Ye<-predict(dd.ld,X)
print(Ye$posterior,digits=3)
LDB$class0<-table(list(obs=catmat,estim=as.vector(Ye$class)))
dimnames(LDB$class)<-list(colnames(Ye$posterior),colnames(Ye$posterior),paste("b",seq(1:ncas),sep=""))
dimnames(LDB$classp)<-list(colnames(Ye$posterior),colnames(Ye$posterior),paste("b",seq(1:ncas),sep=""))
print(LDB$class0)
xx=matrix(0,na,ng)
grnames=colnames(Ye$posterior)
colnames(xx)=grnames
zz=matrix(0,nfa,ng)
grnames=colnames(Ye$posterior)
colnames(zz)=grnames


# beginning bootstrap
if(ncas>0) {
	cat("Case boot - classification matrix\n")
	for (kcas in 1:ncas) {

# tirage au sort pour constituer un echantillon de calibration
		xlist=seq(1,na,by=1)
		samp <- sample(xlist,na,replace=TRUE)
		Xcal=X[samp,]
		Ycal=catmat[samp]
		dd.ldb <- lda(Xcal,t(Ycal)) 
		z=predict(dd.ldb,X)
		xx[,colnames(z$posterior)]=z$posterior
		Yec[samp,,kcas]<-xx[samp,]
		Yev[xlist[-samp],,kcas]<-xx[xlist[-samp],]
		z2=predict(dd.ldb,Z)
		zz[,colnames(z$posterior)]=z2$posterior
		Zec[,,kcas]<-zz
    tt=table(list(obs=catmat[samp],calib=z$class[samp]))
		LDB$class[,colnames(tt),kcas]=tt
    if (option=="extrapolation") {
      tt=table(list(obs=catmat[-samp],verif=z$class[-samp]))
      LDB$classp[rownames(tt),colnames(tt),kcas]=tt
    } else {
      tt=table(list(obs=catnewmat,verif=as.vector(z2$class)))
      LDB$classp[,colnames(tt),kcas]=tt      
    }
		print(paste("boot iter ",kcas))
		print(paste("% tot calib: ",round(100*sum(diag(LDB$class[,,kcas]))/sum(LDB$class[,,kcas]),1)))
		print(paste("% tot verif: ",round(100*sum(diag(LDB$classp[,,kcas]))/sum(LDB$classp[,,kcas]),1)))
	}

	cat("% well classified (median and 95%-conf. interval)\n")
	ccc=rep(0,ncas)
	ccv=rep(0,ncas)
	for (j in 1:ng) {
		cc=LDB$class[j,j,]/apply(LDB$class[j,,],2,sum,na.rm=TRUE)
		ccc=ccc+cc*grsize[j]/na
		cat("calib: ",grnames[j],round(1000*quantile(cc,probs=c(0.025, 0.5, 0.975),na.rm=TRUE))/10,"\n")
		cc=LDB$classp[j,j,]/apply(LDB$classp[j,,],2,sum,na.rm=TRUE)
		ccv=ccv+cc*grsize[j]/na
		cat("verif: ",grnames[j],round(1000*quantile(cc,probs=c(0.025, 0.5, 0.975),na.rm=T))/10,"\n")
	}
	cat("Total % well classified (median and 95%-conf. interval)\n")
	cat("calib: ",round(1000*quantile(ccc,probs=c(0.025, 0.5, 0.975),na.rm=TRUE))/10,"\n")
	cat("verif: ",round(1000*quantile(ccv,probs=c(0.025, 0.5, 0.975),na.rm=TRUE))/10,"\n")

	Ye3<-apply(Yec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=TRUE)
	if (option=="extrapolation") {
	  Ze3<-apply(Yev,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=TRUE)
	} else {
	  Ze3<-apply(Zec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=TRUE)
	}
	for (j in 1:ng) {
# calibration
		LDB$catprob[,(j-1)*3+1]<-Ye3[2,,j]
		LDB$catprob[,(j-1)*3+2]<-Ye3[1,,j]
		LDB$catprob[,(j-1)*3+3]<-Ye3[3,,j]
# verification/prediction
		LDB$catnewprob[,(j-1)*3+1]<-Ze3[2,,j]
		LDB$catnewprob[,(j-1)*3+2]<-Ze3[1,,j]
		LDB$catnewprob[,(j-1)*3+3]<-Ze3[3,,j]
	}
	cat("Summary\n\n")
	Yec=LDB$catprob[,seq(1,ng3,by=3)]
	LDB$catprob[,ng3+1]=grnames[apply(Yec,1,which.max)]
	LDB$clas=table(list(obs=catmat,calib=grnames[apply(Yec,1,which.max)]))	
	print(LDB$clas)	
	if (option=="verification") {
		Yec=LDB$catnewprob[,seq(1,ng3,by=3)]
		LDB$catnewprob[,ng3+1]=grnames[apply(Yec,1,which.max)]
		LDB$clasp=table(list(obs=catnewmat,verif=grnames[apply(Yec,1,which.max)]))
		print(LDB$clasp)
	} else {
	  Yev=LDB$catnewprob[,seq(1,ng3,by=3)]
	  LDB$catnewprob[,ng3+1]=grnames[apply(Yev,1,which.max)]
    LDB$clasp=table(list(obs=catmat,verif=grnames[apply(Yev,1,which.max)]))      
	  print(LDB$clasp)
	}
  
  # output of classification results (for xnewmat)

	Ze3<-apply(Zec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=TRUE)
	LDB$catnewprob=as.data.frame(matrix(NA,nfa,(ng3+1)))
  for (j in 1:ng) {
    LDB$catnewprob[,(j-1)*3+1]<-Ze3[2,,j]
	  LDB$catnewprob[,(j-1)*3+2]<-Ze3[1,,j]
	  LDB$catnewprob[,(j-1)*3+3]<-Ze3[3,,j]
  }
	Zec=LDB$catnewprob[,seq(1,ng3,by=3)]
	LDB$catnewprob[,ng3+1]=grnames[apply(Zec,1,which.max)]
	coln<-rep(NA,ng3)
	for (j in 1:ng) {
		coln[(j-1)*3+1]<-grnames[j]
		coln[(j-1)*3+2]<-paste(grnames[j],'_i',sep="")
		coln[(j-1)*3+3]<-paste(grnames[j],'_s',sep="")
	}
	coln[ng3+1]="assigned"
	colnames(LDB$catprob)<-coln
	colnames(LDB$catnewprob)<-coln
	rownames(LDB$catprob)<-rownames(xmat)
	rownames(LDB$catnewprob)<-rownames(xnewmat)	

	cat("\nImpression de la liste LDB\n")
	print(LDB)
}
if (!is.null(outfile)) {
	dev.off()
	sink()
	sink()
}
return(LDB)
}
