recGLM <-
function(ass,clim,fass,fclim=FALSE,focus="M",outfile=NULL)
{

# Initialisation

library(mgcv)

na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
nfa<-nrow(fass)
mfa<-ncol(fass)
if (identical(na,nc)==FALSE) {stop ("both ass & clim matrices have not same number of rows")}
if (identical(mfa,ma)==FALSE) {stop ("both ass & fass matrices have not same number of columns")}
nfc<-0
mfc<-0
option="extrapolation"
GLM<-list(title="Transfer function estimates by GLM method")
GLM$rmse<-rep(0,mc)
GLM$r2<-rep(0,mc)
mc3=mc*3
GLM$clim<-matrix(0,nc,mc3)
GLM$fclim<-matrix(0,nfa,mc3)
if (is.logical(fclim)==FALSE) 
	{
	nfc<-nrow(fclim)
	mfc<-ncol(fclim)
	if (identical(nfc,nfa)==FALSE) {stop ("both fclim & fass matrices have not same number of rows")}
	option="verification"
	GLM$rmsep<-rep(0,mc)
	GLM$r2p<-rep(0,mc)
	GLM$CE<-rep(0,mc)
	}
if (!is.null(outfile)) 
	{
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
	}

cat("TRANSFER FUNCTION : Generalized Linear Model (GLM)\n")
cat("--------------------------------------------------------------------------------\n")
cat("\nOption : ",option,"\n")

# climatic variable loop

cat("In the verification step, r2p (or reduction of error) compares the predictions to the calibration period climatology\n") 
cat("In the verification step, CE (or coefficient of efficiency) compares the predictions to the verification period climatology\n") 

for (j in 1:mc)
	{
# calibration
	cat("\nDependent Variable : ",colnames(clim[j]),"\n")
	cat("----------------------------------------\n")
	mat<-data.frame(cbind(ass,vd=clim[,j]))
	fmla<-as.formula("vd ~ .")
	if (focus[j]=="L") {cst<-min(clim[,j])*1.01}
	if (focus[j]=="H") {cst<-max(clim[,j])*1.01}
	if (focus[j]=="M") {cst<-mean(clim[,j])}
	w<-1/(clim[,j]-cst)^2
	w<-w/sum(w)
	cat("focus on ",focus[j],"  value = ",cst,"\n")
	glmm<-glm(fmla,data=mat,family=gaussian,weights=w)
	print(summary(glmm))
	mat<-data.frame(ass)
	yp<-predict(glmm,mat,se.fit=TRUE)
	GLM$clim[,j]<-yp$fit
	GLM$rmse[j]<-mean((clim[,j]-yp$fit)^2)^0.5
	GLM$clim[,(j-1)*3+2]<-GLM$rmse[j]
	GLM$clim[,(j-1)*3+3]<-GLM$rmse[j]
	GLM$r2[j]<-1-mean((clim[,j]-yp$fit)^2)/var(clim[,j])
	cat("R2 = ",GLM$r2[j],"  -- RMSE = ",GLM$rmse[j],"calculated on ",na,"  observations\n")
	smin<-min(clim[,j])
	smax<-max(clim[,j])
	plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],"calib"), ylab="Predicted GLM", main="",col="green")
	points(clim[,j],GLM$clim[,(j-1)*3+1],pch=19,col='blue')
# verification
	if (option=="verification")
		{
		mat<-data.frame(fass)
		yp<-predict(glmm,mat,se.fit=TRUE)
		GLM$rmsep[j]<-mean((fclim[,j]-yp$fit)^2)^0.5
		GLM$r2p[j]<-1-mean((fclim[,j]-yp$fit)^2)/var(fclim[,j])
		GLM$CE[j]<-1-mean((fclim[,j]-yp$fit)^2)/var(clim[,j])
		cat("R2p (RE) = ",GLM$r2p[j],"  -- CE = ",GLM$CE[j],"  -- RMSEp = ",GLM$rmsep[j],"calculated on ",nfa,"  observations\n")
		GLM$fclim[,(j-1)*3+1]<-yp$fit
		GLM$fclim[,(j-1)*3+2]<-GLM$rmsep[j]
		GLM$fclim[,(j-1)*3+3]<-GLM$rmsep[j]
		smin<-min(cbind(t(clim[,j]),t(fclim[,j])))
		smax<-max(cbind(t(clim[,j]),t(fclim[,j])))
		plot(c(smin,smax),c(smin,smax),type='l',xlab=paste(colnames(clim)[j],"calib(blue)/verif(red)", ylab="Predicted GLM", main="",col="green"))
		points(clim[,j],GLM$clim[,(j-1)*3+1],pch=19,col="blue")
		points(fclim[,j],GLM$fclim[,(j-1)*3+1],col="red",pch=21)
		}
	else
		{
# extrapolation
		cat("the estimated climate is put in the GLM$fclim matrix\n")
		mat<-data.frame(fass)
		yp<-predict(glmm,mat,se.fit=TRUE)
		GLM$fclim[,(j-1)*3+1]<-yp$fit
		GLM$fclim[,(j-1)*3+2]<-GLM$rmse[j]
		GLM$fclim[,(j-1)*3+3]<-GLM$rmse[j]
		}
	}

coln<-rep(NA,mc3)
for (j in 1:mc) 
	{
	coln[(j-1)*3+1]<-colnames(clim)[j]
	coln[(j-1)*3+2]<-paste(colnames(clim)[j],'_i',sep="")
	coln[(j-1)*3+3]<-paste(colnames(clim)[j],'_s',sep="")
	}
colnames(GLM$clim)<-coln
colnames(GLM$fclim)<-coln
rownames(GLM$clim)<-rownames(clim)
rownames(GLM$fclim)<-rownames(fass)	
cat("\nImpression de la liste GLM\n")
print(GLM)
if (!is.null(outfile))
	{
	dev.off()
	sink()
	}
return(GLM)
}
