respSURF <-
function(ass,clim,nbgrid=10,outfile=NULL)
{
# Initialisation

library(fields)
na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
mc4<-min(mc,4)
mc4s<-mc-mc4
if (identical(na,nc)==FALSE) {stop ("both ass & clim matrices have not same number of rows")}
SURF<-list(title="response surface at max 4 dimensions using a thin plate spline")

if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}

cat("Calculation of the response surface (thin plate spline) \n")
cat("------------------------------------------------------------\n")


# data characteristics
na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
if (identical(nc,na)==FALSE) { stop ("both ass & clim matrices have not same number of rows") }

# construction of the grid
xmin<-apply(ass,2,min)
ymin<-apply(clim,2,min)
xmax<-apply(ass,2,max)
ymax<-apply(clim,2,max)
dy<-(ymax-ymin)/(nbgrid-1)
if (mc4==4) {yg<-expand.grid(y1=seq(ymin[1],ymax[1],dy[1]),y2=seq(ymin[2],ymax[2],dy[2]),y3=seq(ymin[3],ymax[3],dy[3]),y4=seq(ymin[4],ymax[4],dy[4])) }
if (mc4==3) {yg<-expand.grid(y1=seq(ymin[1],ymax[1],dy[1]),y2=seq(ymin[2],ymax[2],dy[2]),y3=seq(ymin[3],ymax[3],dy[3])) }
if (mc4==2) {yg<-expand.grid(y1=seq(ymin[1],ymax[1],dy[1]),y2=seq(ymin[2],ymax[2],dy[2])) }
if (mc4==1) {yg<-expand.grid(y1=seq(ymin[1],ymax[1],dy[1])) }
names(yg)<-colnames(clim)[1:mc4]
npg<-dim(yg)[1]

# building a data.frame with the ref data
mt<-mc+ma
ye<-matrix(0,npg,mt)
SURF$r2<-rep(NA,mt)
data=cbind(clim,ass)

# if the number of climate parameters is >4, we estimate them on the response surface

for (j in 1:mt) 
	{
	cat("\nVariable : ",colnames(data)[j],"\n")
	cat("-------------------------------------------------\n")
	surfm=Tps(data[,1:mc4],data[,j],df=100)
	SURF$r2[j]<-1-var(as.matrix(surfm$residuals),na.rm=TRUE)/var(as.matrix(surfm$y),na.rm=TRUE)
	print(summary(surfm))
	y1<-min(cbind(surfm$y,surfm$fitted.values))
	y2<-max(cbind(surfm$y,surfm$fitted.values))
	plot(c(y1,y2),c(y1,y2), col="red",type='l',xlab=paste("obs / R2=",round(SURF$r2[j]*100)), ylab='pred', main=names(data)[j])
	points(surfm$y,surfm$fitted.values)
	cat("R2 = ",SURF$r2[j],"\n")
	yp<-predict(surfm,yg)
	ye[,j]<-as.matrix(yp)
	}

ye[,1:mc4]<-as.matrix(yg)
r2tot <- mean(SURF$r2,na.rm=TRUE)
cat("R2tot = ",r2tot,"\n")
mina=apply(ass,2,min)
maxa=apply(ass,2,max)
minc=apply(clim,2,min)
maxc=apply(clim,2,max)
deltac=(maxc-minc)
icount<-rep(1,npg)
dd<-rep(NA,npg)
for (i in 1:npg) { dd[i]=min(apply((abs(clim[,1:mc4]-ye[i,1:mc4])/deltac[1:mc4]),1,mean,na.rm=TRUE)) }
seuil=sort(dd)[nc]
if(seuil>2/nbgrid)seuil=2/nbgrid
cat("We retain the gridpoints located at less than ",seuil," than a true point in the 4 clim-variables space \n(after standardisation by the range)\n")
icount[which(dd>=seuil)]=0
icount[which(apply(is.na(ye),1,max)==1)]=0
npgret=sum(icount)
vclim<-matrix(0,npgret,mc)
vass<-matrix(0,npgret,ma)
vclim=ye[which(icount==1),1:mc]
vass=ye[which(icount==1),(mc+1):mt]
for (j in 1:ma) 
	{
	vass[which(vass[,j]<mina[j]),j] = mina[j] 
	vass[which(vass[,j]>maxa[j]),j] = maxa[j] 
	}
colnames(vass)<-colnames(ass)
colnames(vclim)<-colnames(clim)
cat("number of gridpoints kept =",npgret," on a total of ",npg," \n")
if (npgret>0) 
	{
# definition de la fonction histogramme dans la diagonale
	panel.hist <- function(x, ...)
	{
    	usr <- par("usr"); on.exit(par(usr))
    	par(usr = c(usr[1:2], 0, 1.5) )
    	h <- hist(x, breaks=20,plot = FALSE)
    	breaks <- h$breaks; nB <- length(breaks)
    	y <- h$counts; y <- y/max(y)
    	rect(breaks[-nB], 0, breaks[-1], y, col="cyan", ...)
	}
# fin fonction

	pairs(vclim,cex = 0.5,cex.axis=1.2, bg="light blue", main="Position of the gridpoints retained",diag.panel=panel.hist,
      font.labels=1.8,cex.main=1.5)
	}

if (npgret>0) 
	{
	SURF$totass<-rbind(ass[1:nc,],vass[1:npgret,])
	SURF$totclim<-rbind(clim[1:nc,],vclim[1:npgret,])
	}
else 
	{
	SURF$totass<-ass[1:nc,]
	SURF$totclim<-clim[1:nc,]
	}
ntot<-nrow(SURF$totclim)
cat("total number of reference assemblages =",ntot,"\n")
cat("\nImpression de la liste SURF\n")
print(SURF)

if (!is.null(outfile))
	{
	dev.off()
	sink()
	}
return(SURF)
}
