splitdata2 <-
function(ass,clim,coord,nverif,radius=0)
{
# split the datasets (assemblages, climate, coordinates) into calibration and verification subsets
# sort data according to the first pc of the climate
# and randomly select nverif observations in the sorted pc1 with a safety radius (max 10)
# to avoid that verifications observations are too close to the calibrations observations
# so the observations randomly drawn along the gradient of the first PC are marked as "verif" in split$obsrole
# their neighbours (from rank 1 to radius) are marked as "excl" and the other ones as "calib"
# neighbour is in terms of rank and not in term of geographical distance or pc-distance.
# other outputs are the matrices of calibration and verification for assemblages, climate and coordinates
# assc, assv, climc, climv, coordc, coordv
#
# initialisation
x<-as.data.frame(clim)
n<-nrow(x)
m<-ncol(x)
nverif=min(n/2,nverif)
radius=min(10,radius)
xord<-matrix(0,n,m)
vseq=sample(1:n,nverif)
xseq=vseq
if(radius>0) {
  for (k in 1:radius) {
    xseq=c(xseq,vseq-k,vseq+k)
  }
}
xseq[xseq<1]=0
xseq[xseq>n]=0
nv<-length(vseq)

split<-list(title="list of calibration and verification matrices")

# principal component analysis on the climate

pc<-princomp(x,cor=TRUE)
pc1<-pc$scores[,1]
pc1ord<-order(pc1)
split$obsrole=matrix("calib",n,1)
split$obsrole[pc1ord[xseq]]="excl"
split$obsrole[pc1ord[vseq]]="verif"
rownames(split$obsrole)=rownames(clim)
colnames(split$obsrole)="obs_role"

# ass, clim, 

split$climc = clim[split$obsrole=="calib",]
split$climv = clim[split$obsrole=="verif",]
split$assc = ass[split$obsrole=="calib",]
split$assv = ass[split$obsrole=="verif",]
split$coordc = coord[split$obsrole=="calib",]
split$coordv = coord[split$obsrole=="verif",]

return(split)
}
