\name{cartosurf}
\alias{cartosurf}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ map a field on a map, as a surface or as points }
\description{
  If option="s", it uses library(fields) to transform a set of three vectors (longitude, latitude, z) into a surface and plot it with rainbow color codes.
  If option ="p", the points (longitude, latitude, z) are plotted as they are with rainbow color codes.
}
\usage{
cartosurf(lon,lat,champ,limits=NA,zlim=NA,option="p",coldir=1,name="",screen=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lon}{ \code{lon} is the vector of longitude (length n) }
  \item{lat}{ \code{lat} is the vector of latitude (length n) }
  \item{champ}{ \code{champ} is the vector of field to plot with the color code (length n) }
  \item{limits}{ \code{limits} is a four-value vector of limits for longitude (x) and latitude: (minx, maxx, miny, maxy); default is the limits of the data (automatically calculated) }
  \item{zlim}{ \code{zlim} is a two-value vector of limits for field (z): (minz, maxz); default is the limits of the data (automatically calculated) }
  \item{option}{ \code{option} is "s" if a surface is wanted, "p" if points must be kept as they are; "sp" for both surface and points; default="p" }
  \item{coldir}{ \code{coldir} is 1 if the colors must go from blue to red (blue=min, red=max), -1 if the must be reversed (e.g. for precipitation field), if coldir=2: the palette goes from blue to red through white, if coldir=-2: the palette goes from red to blue through white }
  \item{cex}{ \code{cex} relative size of the coloured points: 1 = standard size, any value between 0.1 and 10 }
  \item{name}{ \code{name} is the title for the graphic; name is also used for the pdf-filename if screen=FALSE}
  \item{screen}{ \code{screen} is TRUE if output is displayed onto the screen; FALSE for pdf output}
}
\details{
  This function uses Tps to calculate a surface:
  A thin plate spline is result of minimizing the residual sum of squares subject to a constraint that the function have a certain level of smoothness (or roughness penalty). Roughness is quantified by the integral of squared m-th order derivatives. For one dimension and m=2 the roughness penalty is the integrated square of the second derivative of the function. For two dimensions the roughness penalty is the integral of
  (Dxx(f))**22 + 2(Dxy(f))**2 + (Dyy(f))**22
  (where Duv denotes the second partial derivative with respect to u and v.) Besides controlling the order of the derivatives, the value of m also determines the base polynomial that is fit to the data. The degree of this polynomial is (m-1).
  The smoothing parameter controls the amount that the data is smoothed. In the usual form this is denoted by lambda, the Lagrange multiplier of the minimization problem. Although this is an awkward scale, lambda =0 corresponds to no smoothness constraints and the data is interpolated. lambda=infinity corresponds to just fitting the polynomial base model by ordinary least squares. This estimator is implemented by passing the right generalized covariance function based on radial basis functions to the more general function Krig.
}
\value{
  only graphic output
}
\references{  }
\author{ J. Guiot }
\note{ void
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
  # recommended when the data points are not fairly distributed
  cartosurf(ss700co$LONG,ss700co$LATI,ss700co$ALTI,limits=c(-10,80,35,75))
  # not recommended because the data points are not fairly distributed
  cartosurf(ss700co$LONG,ss700co$LATI,ss700co$ALTI,limits=c(-10,80,35,75),option="s")
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ spline }
\keyword{ kriging }
\keyword{ interpolation }